/*
 * (C) COPYRIGHT 2013 CRZ
 *
 * File Name : mcu_comm_rx_proc.c
 * Author    : POOH
 * Version   : V1.0
 * Date      : 03/22/2013
 */

/* includes */

#include "hw_config.h"
#include "global_types.h"
#include "uart_queue.h"

/* Defines */

typedef enum 
{
    MCU_COMM_RX_STATE_START              = 0,
    MCU_COMM_RX_STATE_HEADER_A1_RECEIVED = 1,
    MCU_COMM_RX_STATE_HEADER_T_RECEIVED  = 2,
    MCU_COMM_RX_STATE_HEADER_A2_RECEIVED = 2,
    MCU_COMM_RX_STATE_ID_OPCODE_RECEIVED   = 3,
    MCU_COMM_RX_STATE_ID_LENGTH_RECEIVED   = 4,
    MCU_COMM_RX_STATE_DATA_RECEIVED        = 5,
    MCU_COMM_RX_STATE_TAIL_DLE_RECEIVED    = 6,
    MCU_COMM_RX_STATE_TAIL_ETX_RECEIVED    = 7,
    MCU_COMM_RX_STATE_CRC1_RECEIVED        = 8,
} Mcu_Comm_Rx_State_E;

/* Variables */

uint16_t  g_micom_id_data;

uint8_t g_mcu_comm_buffer_rx[MCU_COMM_BUFFER_SIZE];
uint8_t g_rx_index = 0;

uint8_t g_rx_data_size = 0;
uint8_t g_rx_data_count = 0;

Mcu_Comm_Rx_State_E g_rx_curr_state = MCU_COMM_RX_STATE_START;

/* Static Functions */

static void initialize_all_data(void)
{
    memset(g_mcu_comm_buffer_rx, 0, MCU_COMM_BUFFER_SIZE);

    g_rx_index = 0;
}

static bool is_packet_receive_done(void)
{
    uint8_t rx_curr_value, i;

    if(true == UartQueue_Bluetooth_Is_Empty())
    {
        return false;
    }

    if(UartQueue_Bluetooth_CurrSize() < MCU_COMM_BUFFER_SIZE)
    {
        initialize_all_data();
        return false;
    }

    rx_curr_value = UartQueue_Bluetooth_DeQueue();
    g_mcu_comm_buffer_rx[g_rx_index++] = rx_curr_value;

    if('A' != g_mcu_comm_buffer_rx[0])
    {
        initialize_all_data();
        return false;
    }

    rx_curr_value = UartQueue_Bluetooth_DeQueue();
    g_mcu_comm_buffer_rx[g_rx_index++] = rx_curr_value;

    if('T' != g_mcu_comm_buffer_rx[1])
    {
        initialize_all_data();
        return false;
    }

    rx_curr_value = UartQueue_Bluetooth_DeQueue();
    g_mcu_comm_buffer_rx[g_rx_index++] = rx_curr_value;

    if('A' != g_mcu_comm_buffer_rx[2])
    {
        initialize_all_data();
        return false;
    }

    for(i = 3; i < MCU_COMM_BUFFER_SIZE; i ++)
    {
        rx_curr_value = UartQueue_Bluetooth_DeQueue();
        g_mcu_comm_buffer_rx[g_rx_index++] = rx_curr_value;
    }

    if((MCU_COMM_TAIL_CR == g_mcu_comm_buffer_rx[MCU_COMM_BUFFER_SIZE - 2])
        && (MCU_COMM_TAIL_LF == g_mcu_comm_buffer_rx[MCU_COMM_BUFFER_SIZE - 1]))
    {
        return true;
    }
    else
    {
        return false;
    }
}

/* Global Functions */

#define MCU_COMM_GET_PACKET_TIME_QUANTUM  900

bool Mcu_Comm_Get_1_Packet(unsigned int timeout_msec)
{
    int i;
    bool check_packet_done = false;

    while(1)
    {
        check_packet_done = is_packet_receive_done();
        if(true == check_packet_done)
        {
            printf("RX: %s\n", g_mcu_comm_buffer_rx);
            for(i = 0; i < g_rx_index; i ++)
            {
                printf("%02X ", g_mcu_comm_buffer_rx[i]);
            }
            printf("\n");

            return true;
        }

        if(0 == timeout_msec)
        {
            return false;
        }

        timeout_msec --;
		usleep(MCU_COMM_GET_PACKET_TIME_QUANTUM);
    }
}

void Mcu_Comm_Receive_Process(void)
{
    /* */

    initialize_all_data();
}

