/*
 * (C) COPYRIGHT 2009 CRZ
 *
 * File Name : main.c
 * Author    : POOH
 * Version   : V1.0
 * Date      : 08/12/2009
 */

/* includes */

#include "hw_config.h"
#include "global_types.h"
#include "uart_queue.h"

/* global variables */

RCC_ClocksTypeDef  rcc_clocks;

/* functions */

void System_Information(void)
{
    printf("IoT AnyThing AT100 Test ... %s %s\n", __DATE__, __TIME__);

    printf("SYSCLK_Frequency = %d\n",rcc_clocks.SYSCLK_Frequency );
    printf("HCLK_Frequency   = %d\n",rcc_clocks.HCLK_Frequency );
    printf("PCLK1_Frequency  = %d\n",rcc_clocks.PCLK1_Frequency );
    printf("PCLK2_Frequency  = %d\n\n",rcc_clocks.PCLK2_Frequency );
}

/**
  * @brief  Configures the used Timers.
  * @param  None
  * @retval None
  */
static void TIM_Configuration(void)
{
    TIM_TimeBaseInitTypeDef  TIM_TimeBaseStructure;

    /* TIM2 configuration */
    TIM_TimeBaseStructure.TIM_Period = 500 - 1;
    TIM_TimeBaseStructure.TIM_Prescaler = 32000 - 1;
    TIM_TimeBaseStructure.TIM_ClockDivision = TIM_CKD_DIV1;    
    TIM_TimeBaseStructure.TIM_CounterMode = TIM_CounterMode_Up;  
    TIM_TimeBaseInit(TIM2, &TIM_TimeBaseStructure);

    /* TIM2 enable counter */
    TIM_Cmd(TIM2, ENABLE);

    /* Immediate load of TIM2 Precaler value */
    TIM_PrescalerConfig(TIM2, 0xEA5F, TIM_PSCReloadMode_Immediate);

    /* Clear TIM2 update pending flag */
    TIM_ClearFlag(TIM2, TIM_FLAG_Update);

    /* Enable TIM2 Update interrupt */
    TIM_ITConfig(TIM2, TIM_IT_Update, ENABLE);
}

void LedControl_forProgramStart(void)
{
#define  LED_CTRL_START_DELAY  30

    LED_On_All();
    Delay(LED_CTRL_START_DELAY);
    LED_Blink_All(5, LED_CTRL_START_DELAY);
    Delay(LED_CTRL_START_DELAY);
    LED_Off_All();
}

bool g_print_menu_control_flag = TRUE;

static void print_menu_command(void)
{
    if(TRUE == g_print_menu_control_flag)
    {
        printf("\n---------------------\n");
        printf("Press menu key\n");
        printf("---------------------\n");
        printf("0> System Information\n");
        printf("---------------------\n");
        printf("1> CR-BT-200 Test - Master Mode\n");
        printf("2> CR-BT-200 Test - Slave Mode\n");
        printf("---------------------\n");
        printf("x> quit\n\n");
    }

    g_print_menu_control_flag = FALSE;
}

/*
 * Name   : main
 * Input  : None
 * Output : None
 * Return : None
 */

int main(void)
{
    uint8_t ch;

    /* System Clocks Configuration */
    RCC_Configuration();
    RCC_GetClocksFreq(&rcc_clocks);

    /* Configure the GPIO ports */
    GPIO_Configuration();

    LED_Off_All();

    /* NVIC configuration */
    NVIC_Configuration();

    /* EXTI configuration */
    EXTI_Configuration();

    /* UART initialization */
    USART1_Init_for_Debug();
    USART2_Init_for_Comm();
    USART3_Init_for_Comm();

    UartQueue_Initialize();
    printf("UartQueue_Initialize() done\n");

    System_Information();

    /* Configure TIM */
    TIM_Configuration();
    printf("TIM_Configuration() done\n");

    /* Setup SysTick Timer for 1 msec interrupts  */
    if (SysTick_Config(rcc_clocks.SYSCLK_Frequency / 1000))
    { 
        /* Capture error */ 
        while (1);
    }
    printf("SysTick_Config() done\n");

    /* Configure I2C */
    I2C_LowLevel_Init();
    printf("I2C_LowLevel_Init() done\n");

    LedControl_forProgramStart();

    Init_BT200();

    while(1)
    {
        print_menu_command();

        if(FALSE == UartQueue_Serial_Is_Empty())
        {
            ch = UartQueue_Serial_DeQueue();
            printf("%c is selected\n\n", (char)ch);

            switch((char)ch)
            {
            case '0':
                System_Information();
                break;

            case '1':
                Test_BT200_Master();
                break;

            case '2':
                Test_BT200_Slave();
                break;
            }

            g_print_menu_control_flag = TRUE;
        }
    }
}

