
/* Includes */

#include "hw_config.h"
#include "global_types.h"

/* Defines */

#define  UART_QUEUE_BUFFER_SIZE  1000

/* Variables */

// UART 2 Queue for Comm
uint16_t  uart_2_queue_comm_buffer[UART_QUEUE_BUFFER_SIZE];
uint32_t  uart_2_queue_comm_point_head = 0;
uint32_t  uart_2_queue_comm_point_tail = 0;
uint32_t  uart_2_queue_comm_size = 0;

// UART 3 Queue for Comm
uint16_t  uart_3_queue_comm_buffer[UART_QUEUE_BUFFER_SIZE];
uint32_t  uart_3_queue_comm_point_head = 0;
uint32_t  uart_3_queue_comm_point_tail = 0;
uint32_t  uart_3_queue_comm_size = 0;

// UART Queue for Serial
uint16_t  uart_queue_serial_buffer[UART_QUEUE_BUFFER_SIZE];
uint32_t  uart_queue_serial_point_head = 0;
uint32_t  uart_queue_serial_point_tail = 0;

/* Static Functions */

static void uart_queue_increase_point_value(uint32_t * data_p)
{
    (* data_p) ++;
    if(UART_QUEUE_BUFFER_SIZE == (* data_p))
    {
        (* data_p) = 0;
    }
}

/* Global Functions */

void UartQueue_Initialize(void)
{
    uart_2_queue_comm_point_head = uart_2_queue_comm_point_tail = 0;
    uart_2_queue_comm_size = 0;

    uart_3_queue_comm_point_head = uart_3_queue_comm_point_tail = 0;
    uart_3_queue_comm_size = 0;
    
    uart_queue_serial_point_head = uart_queue_serial_point_tail = 0;
}

// UART 2 Queue for Comm

bool Uart_2_Queue_Comm_Is_Empty(void)
{
    if(uart_2_queue_comm_point_head == uart_2_queue_comm_point_tail)
    {
        return TRUE;
    }
    return FALSE;
}

uint32_t Uart_2_Queue_Comm_CurrSize(void)
{
    return uart_2_queue_comm_size;
}

void Uart_2_Queue_Comm_EnQueue(uint16_t data)
{
    uart_2_queue_comm_size ++;
    if(UART_QUEUE_BUFFER_SIZE == uart_2_queue_comm_size)
        uart_2_queue_comm_size = 0;

    uart_2_queue_comm_buffer[uart_2_queue_comm_point_head] = data;
    uart_queue_increase_point_value(&uart_2_queue_comm_point_head);
}

uint16_t Uart_2_Queue_Comm_DeQueue(void)
{
    uint16_t retVal = uart_2_queue_comm_buffer[uart_2_queue_comm_point_tail];

    if(0 != uart_2_queue_comm_size)
        uart_2_queue_comm_size --;

    uart_queue_increase_point_value(&uart_2_queue_comm_point_tail);
    return retVal;
}

// UART 3 Queue for Comm

bool Uart_3_Queue_Comm_Is_Empty(void)
{
    if(uart_3_queue_comm_point_head == uart_3_queue_comm_point_tail)
    {
        return TRUE;
    }
    return FALSE;
}

uint32_t Uart_3_Queue_Comm_CurrSize(void)
{
    return uart_3_queue_comm_size;
}

void Uart_3_Queue_Comm_EnQueue(uint16_t data)
{
    uart_3_queue_comm_size ++;
    if(UART_QUEUE_BUFFER_SIZE == uart_3_queue_comm_size)
        uart_3_queue_comm_size = 0;

    uart_3_queue_comm_buffer[uart_3_queue_comm_point_head] = data;
    uart_queue_increase_point_value(&uart_3_queue_comm_point_head);
}

uint16_t Uart_3_Queue_Comm_DeQueue(void)
{
    uint16_t retVal = uart_3_queue_comm_buffer[uart_3_queue_comm_point_tail];

    if(0 != uart_3_queue_comm_size)
        uart_3_queue_comm_size --;

    uart_queue_increase_point_value(&uart_3_queue_comm_point_tail);
    return retVal;
}

// UART Queue for Serial

bool UartQueue_Serial_Is_Empty(void)
{
    if(uart_queue_serial_point_head == uart_queue_serial_point_tail)
    {
        return TRUE;
    }
    return FALSE;
}

void UartQueue_Serial_EnQueue(uint16_t data)
{
    uart_queue_serial_buffer[uart_queue_serial_point_head] = data;
    uart_queue_increase_point_value(&uart_queue_serial_point_head);
}

uint16_t UartQueue_Serial_DeQueue(void)
{
    uint16_t retVal = uart_queue_serial_buffer[uart_queue_serial_point_tail];
    uart_queue_increase_point_value(&uart_queue_serial_point_tail);
    return retVal;
}

